package com.chelvc.cloud.vehicle.client;

import java.util.List;

import com.chelvc.cloud.vehicle.client.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.client.param.CategoryListParam;
import com.chelvc.cloud.vehicle.client.param.CategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.CategoryPagingParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 分类业务操作接口
 *
 * @author Woody
 * @date 2023/5/2
 */
@FeignClient("vehicle")
public interface CategoryClient {
    /**
     * 新增分类
     *
     * @param param 新增参数
     * @return 分类主键
     */
    @PostMapping("/category/addCategory")
    Long addCategory(@RequestBody CategoryModifyParam param);

    /**
     * 修改分类
     *
     * @param id    分类主键
     * @param param 修改参数
     */
    @PutMapping("/category/updateCategory")
    void updateCategory(@RequestParam("id") Long id, @RequestBody CategoryModifyParam param);

    /**
     * 查询分类
     *
     * @param param 查询参数
     * @return 分类分页信息
     */
    @GetMapping("/category/getCategoryPaging")
    Pagination<CategoryDTO> getCategoryPaging(@SpringQueryMap CategoryPagingParam param);

    /**
     * 获取有效分类信息
     *
     * @return 分类信息列表
     */
    @GetMapping("/category/listActiveCategories")
    List<CategoryDTO> listActiveCategories(@SpringQueryMap CategoryListParam param);

    /**
     * 获取有效分类信息
     *
     * @return 分类信息列表
     */
    @GetMapping("/category/listCategory")
    List<CategoryDTO> listCategory(@SpringQueryMap CategoryListParam param);

    /**
     * 获取推荐分类信息
     *
     * @param size 分类个数
     * @return 分类信息列表
     */
    @GetMapping("/category/listRecommendCategories")
    List<CategoryDTO> listRecommendCategories(@RequestParam("size") int size);


    /**
     * 删除分类
     *
     * @param id 分类主键
     */
    @DeleteMapping("/category")
    void deleteCategory(@RequestParam("id") Long id);

    /**
     * 获取分类列表
     *
     * @return 分类信息
     */
    @GetMapping("/category/getRootCategories")
    List<CategoryDTO> getRootCategories(@SpringQueryMap CategoryListParam param);

    /**
     * 获取一级分类
     *
     * @return 分类信息
     */
    @GetMapping("/category/getParentCategories")
    List<CategoryDTO> getParentCategories(@SpringQueryMap CategoryListParam param);
}
