package com.chelvc.cloud.vehicle.client;

import com.chelvc.cloud.vehicle.client.dto.MerchantAuthDTO;
import com.chelvc.cloud.vehicle.client.param.MerchantAuthModifyParam;
import com.chelvc.cloud.vehicle.client.param.MerchantAuthPagingParam;
import com.chelvc.framework.common.model.Pagination;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 商家认证业务接口
 *
 * @author liude
 * @data 2023/12/7
 */
@FeignClient("vehicle")
public interface MerchantAuthClient {

    /**
     * 新增商家认证
     *
     * @param param 新增参数
     * @return 主键
     */
    @PostMapping("/merchantAuth/changeSystemInformFlag")
    Long addMerchantAuth(@RequestBody MerchantAuthModifyParam param);

    /**
     * 修改商家认证
     *
     * @param id    主键
     * @param param 修改参数
     */
    @PutMapping("/merchantAuth/updateMerchantAuth")
    void updateMerchantAuth(@RequestParam("id") Long id, @RequestBody MerchantAuthModifyParam param);

    /**
     * 查询商家认证
     *
     * @param param 查询参数
     * @return 商家认证分页信息
     */
    @GetMapping("/merchantAuth/getMerchantAuthPaging")
    Pagination<MerchantAuthDTO> getMerchantAuthPaging(@SpringQueryMap MerchantAuthPagingParam param);

    /**
     * 查询商家认证信息
     *
     * @return
     */
    @GetMapping("/merchantAuth/queryById")
    MerchantAuthDTO queryById();

    /**
     * 商家认证是否通过
     *
     * @param id    主键ID
     * @param state 0-拒绝 1-通过
     */
    @PostMapping("/merchantAuth/certificationAudit")
    void certificationAudit(@RequestParam("id") Long id, @RequestParam("state") String state,
                            @RequestParam("message") String message);
}
