package com.chelvc.cloud.vehicle.server.controller;

import com.chelvc.cloud.vehicle.client.dto.MerchantAuthDTO;
import com.chelvc.cloud.vehicle.client.param.MerchantAuthModifyParam;
import com.chelvc.cloud.vehicle.client.param.MerchantAuthPagingParam;
import com.chelvc.cloud.vehicle.server.service.MerchantAuthService;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 商家认证业务接口
 *
 * @author liude
 * @data 2023/12/7
 */
@RestController
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class MerchantAuthController {

    private final MerchantAuthService merchantAuthService;

    /**
     * 新增商家认证
     *
     * @param param 新增参数
     * @return 主键
     */
    @PostMapping("/merchantAuth/changeSystemInformFlag")
    public Long addMerchantAuth(@RequestBody MerchantAuthModifyParam param) {
        return merchantAuthService.addMerchantAuth(param);
    }

    /**
     * 修改商家认证
     *
     * @param id    主键
     * @param param 修改参数
     */
    @PutMapping("/merchantAuth/updateMerchantAuth")
    public void updateMerchantAuth(@RequestParam("id") Long id, @RequestBody MerchantAuthModifyParam param) {
        merchantAuthService.updateMerchantAuth(id, param);
    }

    /**
     * 查询商家认证
     *
     * @param param 查询参数
     * @return 商家认证分页信息
     */
    @GetMapping("/merchantAuth/getMerchantAuthPaging")
    public Pagination<MerchantAuthDTO> getMerchantAuthPaging(MerchantAuthPagingParam param) {
        return merchantAuthService.getMerchantAuthPaging(param);
    }

    /**
     * 查询商家认证信息
     *
     * @return
     */
    @GetMapping("/merchantAuth/queryById")
    public MerchantAuthDTO queryById() {
        return merchantAuthService.queryById();
    }

    /**
     * 商家认证是否通过
     *
     * @param id    主键ID
     * @param state 0-拒绝 1-通过
     */
    @PostMapping("/merchantAuth/certificationAudit")
    public void certificationAudit(@RequestParam("id") Long id, @RequestParam("state") String state,
                                   @RequestParam("message") String message) {
        merchantAuthService.certificationAudit(id, state, message);
    }
}
