package com.chelvc.cloud.vehicle.server.service;

import com.chelvc.cloud.vehicle.client.dto.OmsOrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderReturnApplyIdsDTO;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderReturnApply;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;

import java.util.List;

/**
 * 订单退货申请Service接口
 *
 * @author liude
 * @date 2023-11-08
 */
public interface OmsOrderReturnApplyService
{
    /**
     * 查询订单退货申请
     *
     * @param id 订单退货申请主键
     * @return 订单退货申请
     */
    public OmsOrderReturnApply selectOmsOrderReturnApplyById(Long id);

    /**
     * 查询订单退货申请列表
     *
     * @param omsOrderReturnApply 订单退货申请
     * @return 订单退货申请集合
     */
    public List<OmsOrderReturnApply> selectOmsOrderReturnApplyList(OmsOrderReturnApply omsOrderReturnApply);

    /**
     * 新增订单退货申请
     *
     * @param omsOrderReturnApply 订单退货申请
     * @return 结果
     */
    public int insertOmsOrderReturnApply(OmsOrderReturnApply omsOrderReturnApply);

    /**
     * 修改订单退货申请
     *
     * @param omsOrderReturnApply 订单退货申请
     * @return 结果
     */
    public int updateOmsOrderReturnApply(OmsOrderReturnApply omsOrderReturnApply);

    /**
     * 批量删除订单退货申请
     *
     * @param ids 需要删除的订单退货申请主键集合
     * @return 结果
     */
    public int deleteOmsOrderReturnApplyByIds(Long[] ids);

    /**
     * 删除订单退货申请信息
     *
     * @param id 订单退货申请主键
     * @return 结果
     */
    public int deleteOmsOrderReturnApplyById(Long id);

    /**
     * 查看商家端申请退款订单
     * @param merchantId
     * @param paging
     * @return
     */
    Pagination<OmsOrderReturnApplyIdsDTO> listOrderReturnApply(Long merchantId, Paging paging);

    /**
     * 查看退款订单数
     * @param userId
     * @return
     */
    Integer queryReturnOrderNum(Long userId);

    OmsOrderReturnApplyDTO getOrderReturnApplyInfo(Long id);
}
