package com.chelvc.cloud.vehicle.server.service.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.lock.annotation.Lock4j;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.vehicle.client.constant.DynamicStatusType;
import com.chelvc.cloud.vehicle.client.constant.ExamineType;
import com.chelvc.cloud.vehicle.client.constant.RecordType;
import com.chelvc.cloud.vehicle.client.dto.DynamicCommentDTO;
import com.chelvc.cloud.vehicle.client.dto.DynamicEvaluateRecordDTO;
import com.chelvc.cloud.vehicle.client.dto.EvaluateDTO;
import com.chelvc.cloud.vehicle.client.param.AddDynamicCommentParam;
import com.chelvc.cloud.vehicle.client.param.QueryAdminDynamicParam;
import com.chelvc.cloud.vehicle.client.param.QueryDynamicCommentParam;
import com.chelvc.cloud.vehicle.server.dao.DynamicCommentMapper;
import com.chelvc.cloud.vehicle.server.entity.DynamicComment;
import com.chelvc.cloud.vehicle.server.service.DynamicCommentService;
import com.chelvc.cloud.vehicle.server.service.DynamicContentService;
import com.chelvc.cloud.vehicle.server.service.DynamicEvaluateRecordService;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @PACKAGE_NAME: net.yeeu.collision.dynamic.service.impl
 * @NAME: DynamicCommentServiceImpl
 * @USER: igl
 * @DATE: 2023/8/8 18:53
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class DynamicCommentServiceImpl implements DynamicCommentService {
    private final DynamicCommentMapper baseMapper;
    private final DynamicContentService dynamicContentService;
    private final UserClient userClient;
    DynamicEvaluateRecordService dynamicEvaluateRecordService;

    //private final ContentAuditHelper contentAuditHelper;

    /**
     * 查询动态评论列表
     */
    @Override
    public Pagination<DynamicCommentDTO> queryPageList(QueryDynamicCommentParam param, Integer pageNum,
                                                       Integer pageSize) {
        LambdaQueryWrapper<DynamicComment> lqw = Wrappers.lambdaQuery();
        lqw.eq(DynamicComment::getDynamicId, param.getDynamicId());
        lqw.eq(DynamicComment::getParentId, param.getParentId());
        lqw.eq(DynamicComment::getPlatformStatus, 0);
        Long userId = param.getUserId();
        /*lqw.and(wq -> wq
                .eq(DynamicComment::getExamineStatus, 1)
                .or(wq1 -> wq1
                        .eq(DynamicComment::getExamineStatus, 0)
                        .eq(DynamicComment::getUserId, userId)));*/
        lqw.orderByDesc(DynamicComment::getCreateTime);
        Page<DynamicComment> page = baseMapper.selectPage(
                new Page<>(pageNum, pageSize), lqw);
        List<DynamicComment> result = page.getRecords();
        DynamicEvaluateRecordService service = getContentEvaluateRecordService();
        if (result != null && result.size() > 0) {
            List<DynamicCommentDTO> records = ObjectUtils.copying(result,
                    DynamicCommentDTO.class);
            List<Long> userIds = records.stream().map(DynamicCommentDTO::getUserId).collect(
                    Collectors.toList());
            List<UserDTO> users = this.userClient.listUsers(userIds);
            Map<Long, UserDTO> map = users.stream().collect(Collectors.toMap(UserDTO::getId, user -> user));
            records.forEach(e -> {
                e.setUserDTO(map.get(e.getUserId()));
                DynamicEvaluateRecordDTO dto = service.queryOperaterType(e.getId(),
                        userId, RecordType.COMMENT.getValue());
                if (dto == null) {
                    e.setClickStatus(0);
                } else {
                    e.setClickStatus(dto.getOperaterType());
                }
            });
            return Pagination.<DynamicCommentDTO>builder().total(page.getTotal()).pages(page.getPages())
                    .records(records).build();
        }
        return new Pagination<>();
    }

    public DynamicEvaluateRecordService getContentEvaluateRecordService() {
        this.dynamicEvaluateRecordService = SpringUtil.getBean(DynamicEvaluateRecordService.class);
        return dynamicEvaluateRecordService;
    }

    @Override
    public Pagination<DynamicCommentDTO> queryAdminPageList(QueryAdminDynamicParam param) {
        Paging paging = param.getPaging();
        Page<DynamicCommentDTO> page = new Page<>(paging.getNumber(), paging.getSize());
        IPage<DynamicCommentDTO> result = null;
        result = baseMapper
                .queryAll(page, queryManagerWrapper(param));
        IPage<DynamicCommentDTO> resultDto = getResult(result);
        return Pagination.<DynamicCommentDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(resultDto.getRecords()).build();
    }

    public IPage<DynamicCommentDTO> getResult(IPage<DynamicCommentDTO> result) {
        if (result == null) {
            return null;
        }
        List<DynamicCommentDTO> records = result.getRecords();
        if (records != null && !records.isEmpty()) {
            List<Long> userIds = records.stream().map(DynamicCommentDTO::getUserId).collect(
                    Collectors.toList());
            List<UserDTO> users = this.userClient.listUsers(userIds);
            Map<Long, UserDTO> longUserBaseInfoMap = users.stream().collect(Collectors.toMap(UserDTO::getId,
                    user -> user));
            log.info("远程查询用户信息成功:{}", longUserBaseInfoMap);
            records.forEach(e -> {
                e.setUserDTO(longUserBaseInfoMap.get(e.getUserId()));
            });
        }
        return result;
    }

    private QueryWrapper<DynamicComment> queryManagerWrapper(
            QueryAdminDynamicParam param) {
        QueryWrapper<DynamicComment> lqw = Wrappers.query();
        lqw.eq("t1.deleted", 0);
        lqw.like(StringUtils.isNotBlank(param.getKeyWord()), "t1.content", param.getKeyWord());
        lqw.ge(StringUtils.isNotBlank(param.getStartTime()), "t1.create_time", param.getStartTime());
        lqw.le(StringUtils.isNotBlank(param.getEndTime()), "t1.create_time", param.getEndTime());
        lqw.eq(param.getExamineStatus() != null, "t1.examine_status", param.getExamineStatus());
        lqw.eq(param.getExamineResult() != null, "t1.examine_result", param.getExamineResult());
        lqw.eq(param.getPlatformStatus() != null, "t1.platform_status", param.getPlatformStatus());
        lqw.orderByDesc("t1.create_time", "t1.id");
        return lqw;
    }

    /**
     * 新增动态评论
     */
    @Override
    public void insertByBo(AddDynamicCommentParam param, Long userId) {
        DynamicComment add = ObjectUtils.copying(param, DynamicComment.class);
        add.setUserId(userId);
        boolean b = baseMapper.insert(add) > 0;
        if (!b) {
            throw new ResourceUnavailableException("新增评论失败");
        }
        //异步处理审核
        //contentAuditHelper.asyncCommentHandle(add);
        //暂时不审核
        addParentComment(param.getParentId());
        dynamicContentService.addComment(param.getDynamicId());
    }

    private void addParentComment(Long parentId) {
        if (parentId == 0) {
            return;
        }
        DynamicComment DynamicComment = baseMapper.selectById(parentId);
        if (DynamicComment == null) {
            return;
        }
        int i = baseMapper.addComment(DynamicComment.getId());
        if (i != 1) {
            throw new ResourceUnavailableException("新增评论失败");
        }
        addParentComment(DynamicComment.getParentId());
    }

    @Override
    @Lock4j(keys = {"#contentId"})
    public void addNum(Integer dynamicStatus, Long contentId) {
        if (Objects.equals(DynamicStatusType.GOOD.getValue(), dynamicStatus)) {
            int i = baseMapper.addGoodNum(contentId);
            if (i != 1) {
                throw new ResourceUnavailableException("点赞失败");
            }
        } else if (Objects.equals(DynamicStatusType.BAD.getValue(), dynamicStatus)) {
            int i = baseMapper.addBadNum(contentId);
            if (i != 1) {
                throw new ResourceUnavailableException("D踩一踩失败");
            }
        }
    }

    @Override
    @Lock4j(keys = {"#contentId"})
    public void reduceNum(Integer dynamicStatus, Long contentId) {
        if (Objects.equals(DynamicStatusType.GOOD.getValue(), dynamicStatus)) {
            int i = baseMapper.reduceGoodNum(contentId);
            if (i != 1) {
                throw new ResourceUnavailableException("取消点赞失败");
            }
        } else if (Objects.equals(DynamicStatusType.BAD.getValue(), dynamicStatus)) {
            int i = baseMapper.reduceBadNum(contentId);
            if (i != 1) {
                throw new ResourceUnavailableException("取消踩一踩失败");
            }
        }
    }

    @Override
    public void examine(Long id, ExamineType examine) {
        DynamicComment DynamicComment = baseMapper.selectById(id);
        if (DynamicComment == null) {
            throw new ResourceUnavailableException("评论不存在");
        }
        LambdaUpdateWrapper<DynamicComment> lwq = Wrappers.lambdaUpdate();
        lwq.eq(com.chelvc.cloud.vehicle.server.entity.DynamicComment::getId, id);
        lwq.set(com.chelvc.cloud.vehicle.server.entity.DynamicComment::getExamineStatus, 1);
        if (ExamineType.REFUSE.equals(examine)) {
            lwq.set(com.chelvc.cloud.vehicle.server.entity.DynamicComment::getPlatformStatus, 1);
        }
        int update = baseMapper.update(null, lwq);
        if (update != 1) {
            throw new ResourceUnavailableException("操作失败");
        }
        if (ExamineType.PASS.equals(examine)) {
            DatabaseContextHolder.transactional((Executor) () -> {
                addParentComment(DynamicComment.getParentId());
                dynamicContentService.addComment(DynamicComment.getDynamicId());
            });
        }
    }

    @Override
    public void autoAudit(Long id, int platformStatus, int examineResult) {
        LambdaUpdateWrapper<DynamicComment> lwq = Wrappers.lambdaUpdate();
        lwq.set(DynamicComment::getPlatformStatus, platformStatus);
        lwq.set(DynamicComment::getExamineStatus, 1);
        lwq.set(DynamicComment::getExamineResult, examineResult);
        lwq.set(DynamicComment::getUpdateTime, LocalDateTime.now());
        lwq.eq(DynamicComment::getId, id);
        int update = baseMapper.update(null, lwq);
        if (update != 1) {
            throw new ResourceUnavailableException("自动审核失败");
        }
        if (ExamineType.PASS.getValue() == examineResult) {
            DynamicComment DynamicComment = baseMapper.selectById(id);
            addParentComment(DynamicComment.getParentId());
            dynamicContentService.addComment(DynamicComment.getDynamicId());
        }
    }

    @Override
    public DynamicCommentDTO queryById(Long id) {
        DynamicComment dynamicComment = baseMapper.selectById(id);
        if (dynamicComment == null) {
            return null;
        }
        return ObjectUtils.copying(dynamicComment, DynamicCommentDTO.class);
    }

    @Override
    public Integer queryNewInformNum(Long userId) {
        return baseMapper.queryNewInformNum(userId);
    }

    @Override
    public Pagination<EvaluateDTO> selectEvaluateListByUserId(Long userId, Paging paging) {
        int pageNum = (paging.getNumber() - 1) * paging.getSize();
        if(pageNum < 0){
            return null;
        }
        Long total = baseMapper.selectTotalByUserId(userId);
        List<EvaluateDTO> copy = baseMapper.selectEvaluateListByUserId(userId, pageNum, paging.getSize());
        if (copy != null && copy.size() > 0) {
            for (EvaluateDTO evaluateDTO : copy) {
                Integer contentType = evaluateDTO.getContentType();
                if (contentType == 1) {
                    evaluateDTO.setDynamicContentDTO(dynamicContentService.queryById(evaluateDTO.getContentId(),
                            evaluateDTO.getUserId()));
                } else {
                    DynamicCommentDTO dynamicCommentDTO = queryById(evaluateDTO.getContentId());
                    if (dynamicCommentDTO != null) {
                        evaluateDTO.setDynamicContentDTO(dynamicContentService.queryById(dynamicCommentDTO.getDynamicId(), evaluateDTO.getUserId()));
                    }
                    evaluateDTO.setDynamicCommentDTO(dynamicCommentDTO);
                }
            }
        }
        return Pagination.<EvaluateDTO>builder().total(total).pages(getPage((long) paging.getSize(), total))
                .records(copy).build();
    }

    @Override
    public void changeEvaluateInformFlag(Long userId) {
        DynamicEvaluateRecordService service = getContentEvaluateRecordService();
        service.changeEvaluateInformFlag(userId);
        LambdaUpdateWrapper<DynamicComment> lwq = Wrappers.lambdaUpdate();
        lwq.eq(DynamicComment::getTargetUserId, userId);
        lwq.eq(DynamicComment::getFlag, 0);
        lwq.set(DynamicComment::getFlag, 1);
        baseMapper.update(null, lwq);
    }

    public long getPage(Long size, Long total){
        if (size == 0L) {
            return 0L;
        } else {
            long pages = total / size;
            if (total % size != 0L) {
                ++pages;
            }

            return pages;
        }
    }
}
