package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.client.dto.HelpCategoryDTO;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryModifyParam;
import com.chelvc.cloud.vehicle.client.param.HelpCategoryPagingParam;
import com.chelvc.cloud.vehicle.server.copier.HelpCategoryCopier;
import com.chelvc.cloud.vehicle.server.dao.HelpCategoryMapper;
import com.chelvc.cloud.vehicle.server.entity.HelpCategory;
import com.chelvc.cloud.vehicle.server.service.HelpCategoryService;
import com.chelvc.cloud.vehicle.server.service.HelpService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 帮助分类Service业务层处理
 *
 * @author liude
 * @date 2023-11-08
 */
@Service
@RequiredArgsConstructor(onConstructor = @__({@Autowired, @Lazy}))
public class HelpCategoryServiceImpl extends ServiceImpl<HelpCategoryMapper,HelpCategory> implements
        HelpCategoryService

{
    private final HelpService helpService;
    /**
     * 查询帮助分类
     *
     * @param id 帮助分类主键
     * @return 帮助分类
     */
    @Override
    public HelpCategoryDTO selectHelpCategoryById( Long id)
    {
        return HelpCategoryCopier.INSTANCE.copying(this.getById(id));
    }

    /**
     * 查询帮助分类列表
     *
     * @param param 帮助分类
     * @return 帮助分类
     */
    @Override
    public List<HelpCategoryDTO> selectHelpCategoryList(@NonNull HelpCategoryModifyParam param)
    {
        List<HelpCategory> helpCategories = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getName()),HelpCategory::getName,param.getName())
                .eq(StringUtils.notEmpty(param.getShowStatus()),HelpCategory::getShowStatus,param.getShowStatus())
                .eq(StringUtils.notEmpty(param.getCategoryId()),HelpCategory::getId,param.getCategoryId())
                .orderByDesc(HelpCategory::getSort)
                .list();
        List<HelpCategoryDTO> helpCategoryList = HelpCategoryCopier.INSTANCE.copying(helpCategories);
        helpCategoryList.forEach(item ->{
            item.setHelpList(this.helpService.helpList(item.getId()));
        });
        return helpCategoryList;
    }

    /**
     * 新增帮助分类
     *
     * @param param 帮助分类
     * @return 结果
     */
    @Override
    public void insertHelpCategory(@NonNull HelpCategoryModifyParam param)
    {
        HelpCategory helpCategory = HelpCategoryCopier.INSTANCE.copying(param);
        Long userId = SessionContextHolder.getId();
        helpCategory.setCreateTime(new Date());
        helpCategory.setUpdater(userId);
        helpCategory.setUpdateTime(new Date());
        helpCategory.setCreator(userId);
        this.save(helpCategory);
    }

    /**
     * 修改帮助分类
     *
     * @param param 帮助分类
     * @return 结果
     */
    @Override
    public void updateHelpCategory(@NonNull Long id, @NonNull HelpCategoryModifyParam param)
    {
        HelpCategory helpCategory = AssertUtils.available(this.getById(id), "该分类不存在");
        HelpCategoryCopier.INSTANCE.copying(param,helpCategory);
        Long userId = SessionContextHolder.getId();
        helpCategory.setUpdater(userId);
        helpCategory.setUpdateTime(new Date());
        this.updateById(helpCategory);
    }



    /**
     * 删除帮助分类信息
     *
     * @param id 帮助分类主键
     * @return 结果
     */
    @Override
    public void deleteHelpCategoryById(@NonNull Long id)
    {
        HelpCategory helpCategory = AssertUtils.available(this.getById(id), "该分类不存在");
        this.deleteHelpCategoryById(id);
    }


    @Override
    public Pagination<HelpCategoryDTO> getHelpCategoryPaging(HelpCategoryPagingParam param){
        Page<HelpCategory> page = this.lambdaQuery()
                .like(StringUtils.notEmpty(param.getTitle()),HelpCategory::getName,param.getTitle())
                .orderByDesc(HelpCategory::getShowStatus)
                .orderByDesc(HelpCategory::getSort)
                .page(DatabaseContextHolder.page(param.getPaging()));
        return DatabaseContextHolder.pagination(page, HelpCategoryCopier.INSTANCE::copying);
    }


    /**
     * 查询帮助分类列表
     *
     * @return 帮助分类
     */
    @Override
    public List<HelpCategoryDTO> selectHelpCategories()
    {
        List<HelpCategory> helpCategories = this.lambdaQuery()
                .eq(HelpCategory::getShowStatus,0)
                .orderByDesc(HelpCategory::getSort)
                .list();
        List<HelpCategoryDTO> helpCategoryList = HelpCategoryCopier.INSTANCE.copying(helpCategories);
        return helpCategoryList;
    }
}
