package com.chelvc.cloud.vehicle.server.service.impl;

import com.chelvc.cloud.vehicle.client.constant.SystemInformType;
import com.chelvc.cloud.vehicle.client.dto.*;
import com.chelvc.cloud.vehicle.server.service.*;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class InformServiceImpl implements InformService {
    private final OmsOrderOperateHistoryService omsOrderOperateHistoryService;
    private final UserCouponService userCouponService;
    private final OmsOrderPayHistoryService omsOrderPayHistoryService;
    private final CommentService commentService;
    private final OmsOrderReturnApplyService omsOrderReturnApplyService;
    private final DynamicCommentService dynamicCommentService;

    @Override
    public Pagination<OmsOrderOperateHistoryDTO> selectOmsOrderOperateHistoryListByUserId(Long userId, Paging paging) {
        return omsOrderOperateHistoryService.selectOmsOrderOperateHistoryListByUserId(userId, paging);
    }

    @Override
    public void changeSystemInformFlag(SystemInformType type, Long userId) {
        switch (type){
            case PAY:
                omsOrderPayHistoryService.changeUserPayInformFlag(userId);
                break;
            case ORDER:
                omsOrderOperateHistoryService.changeUserOrderInformFlag(userId);
                break;
            case COUPON:
                userCouponService.changeUserCouponInformFlag(userId);
                break;
            case COMMENT:
                commentService.changeCommentInformFlag(userId);
                break;
            case EVALUATE:
                dynamicCommentService.changeEvaluateInformFlag(userId);
                break;
        }
    }

    @Override
    public NewInformDTO getNewInform(Long userId) {
        NewInformDTO newInformDTO = new NewInformDTO();
        Integer evaluate = dynamicCommentService.queryNewInformNum(userId);
        newInformDTO.setEvaluate(evaluate);
        Integer order = omsOrderOperateHistoryService.queryNewInformNum(userId);
        newInformDTO.setOrder(order);
        Integer coupon = userCouponService.queryNewInformNum(userId);
        newInformDTO.setCoupon(coupon);
        Integer pay = omsOrderPayHistoryService.queryNewInformNum(userId);
        newInformDTO.setPay(pay);
        Integer comment = commentService.queryNewInformNum(userId);
        newInformDTO.setComment(comment);
        return newInformDTO;
    }

    @Override
    public NewMerchantInformDTO getMerchantNewInform(Long merchantId) {
        NewMerchantInformDTO dto = new NewMerchantInformDTO();
        Integer orderNum = omsOrderReturnApplyService.queryReturnOrderNum(merchantId);
        dto.setOrderNum(orderNum);
        return dto;
    }

    @Override
    public Pagination<OmsOrderReturnApplyIdsDTO> listOrderReturnApply(Long userId, Paging paging) {
        return omsOrderReturnApplyService.listOrderReturnApply(userId, paging);
    }

    @Override
    public Pagination<OmsOrderDTO> selectOmsOrderPayHistoryListByUserId(Long merchantId, Paging paging) {
        return omsOrderPayHistoryService.selectOmsOrderPayHistoryListByUserId(merchantId, paging);
    }

    @Override
    public Pagination<EvaluateDTO> selectEvaluateListByUserId(Long userId, Paging paging) {
        return dynamicCommentService.selectEvaluateListByUserId(userId, paging);
    }

    @Override
    public Pagination<CommentDTO> selectCommentListByUserId(Long userId, Paging paging) {
        return commentService.selectCommentListByUserId(userId, paging);
    }

    @Override
    public Pagination<UserCouponDTO> selectUserCouponListByUserId(Long userId, Paging paging) {
        return userCouponService.selectUserCouponListByUserId(userId, paging);
    }
}
